use crate::{
    entity::prelude::*,
    errors::NoUserInfo,
    handlers::{markups::deletion_markup, utils::package_handler, MainDialogue, State},
};
use sea_orm::prelude::*;
use teloxide::{adaptors::Throttle, prelude::*};

/// Gets the master password, deletes the accounts and the master password from DB.
/// Although it doesn't use the master password, we get it to be sure that it's the user who used that command
async fn get_master_pass(
    bot: Throttle<Bot>,
    msg: Message,
    db: DatabaseConnection,
    dialogue: MainDialogue,
    _master_pass: String,
) -> crate::Result<()> {
    dialogue.exit().await?;
    let user_id = msg.from().ok_or(NoUserInfo)?.id.0;
    Account::delete_all(user_id, &db).await?;
    MasterPass::delete_by_id(user_id).exec(&db).await?;
    bot.send_message(msg.chat.id, "Everything was deleted")
        .reply_markup(deletion_markup())
        .await?;
    Ok(())
}

/// Handles /delete_all command
pub async fn delete_all(
    bot: Throttle<Bot>,
    msg: Message,
    dialogue: MainDialogue,
) -> crate::Result<()> {
    let previous = bot
        .send_message(
            msg.chat.id,
            "Send master password to delete EVERYTHING.\nTHIS ACTION IS IRREVERSIBLE",
        )
        .await?;
    dialogue
        .update(State::GetMasterPass(package_handler(
            get_master_pass,
            previous,
        )))
        .await?;
    Ok(())
}
